#!/usr/bin/ruby

require 'getoptlong' # keep it simple for the demo

# This is the list of different tasks the end user gets to choose from.
ACTIONS = {
  1 => {name: 'eggs', desc: 'Make some nice eggs.', func: 'scramble_eggs'},
  2 => {name: 'milk', desc: 'Serve some milk.', func: 'heat_milk'},
  3 => {name: 'fruits', desc: 'Apples & Bananas!', func: 'slice_fruits'},
}

def get_action_id(opts, actions)
  # Get the provided command line argument. This would not work if more cmd
  # line arguments were possible; just for the sake of the example.
  _, aid = opts.get

  if aid == nil # no cmd line argument provided, show menu
    actions.each do |_aid, value|
      puts "%i: %s [aid '%s' or '%s']" % [_aid, actions[_aid][:desc], _aid, actions[_aid][:name]]
    end
    print "Please choose: "
    aid = gets.chomp
  else
    # Check if provided cmd line argument is the action name, not id
    name2aid = {} # temporary mapping 
    actions.each do |_aid, value|
      name2aid[actions[_aid][:name]] = _aid
    end
    if name2aid.key?(aid) # cmd line argument provided as name
      aid = name2aid[aid].to_s
    end
  end

  # Ensure that the returned aid is indeed a valid key of the "actions" hash.
  if aid.to_i.to_s != aid or !actions.has_key?(aid.to_i)
    abort("Action '#{aid}' is invalid!")
  end
  aid = aid.to_i
  return aid
end

# Task
def scramble_eggs(opts)
  puts 'Making eggs ...'
end

# Task
def heat_milk(opts)
  puts 'Heating milk ...'
end

# Task
def slice_fruits(opts)
  puts 'Slicing fruits ...'
end

# "main"
opts = GetoptLong.new(
  ['--aid', GetoptLong::REQUIRED_ARGUMENT],
)
# Choose task.
aid = get_action_id(opts, ACTIONS)
# Ruby arcane to call the named function.
# In other languages, a function pointer would do the trick.
send(ACTIONS[aid][:func], opts)
